@@ -1,7 +1,10 @@
-/* $Id: inttree.h 1973 2007-09-21 16:10:16Z peter $ */
 #ifndef YASM_INTTREE_H
 #define YASM_INTTREE_H
 
+#ifndef YASM_LIB_DECL
+#define YASM_LIB_DECL
+#endif
+
 /* The interval_tree.h and interval_tree.cc files contain code for 
  * interval trees implemented using red-black-trees as described in
  * the book _Introduction_To_Algorithms_ by Cormen, Leisserson, 
@@ -45,14 +48,22 @@ typedef struct IntervalTree {
     unsigned int recursionNodeStackTop;
 } IntervalTree;
 
+YASM_LIB_DECL
 IntervalTree *IT_create(void);
+YASM_LIB_DECL
 void IT_destroy(IntervalTree *);
+YASM_LIB_DECL
 void IT_print(const IntervalTree *);
+YASM_LIB_DECL
 void *IT_delete_node(IntervalTree *, IntervalTreeNode *, long *low,
                      long *high);
+YASM_LIB_DECL
 IntervalTreeNode *IT_insert(IntervalTree *, long low, long high, void *data);
+YASM_LIB_DECL
 IntervalTreeNode *IT_get_predecessor(const IntervalTree *, IntervalTreeNode *);
+YASM_LIB_DECL
 IntervalTreeNode *IT_get_successor(const IntervalTree *, IntervalTreeNode *);
+YASM_LIB_DECL
 void IT_enumerate(IntervalTree *, long low, long high, void *cbd,
                   void (*callback) (IntervalTreeNode *node, void *cbd));
 
